<?php
$curr_directory = __DIR__;
require_once "$curr_directory/../vendor/autoload.php";

$min_urls = 50;
$max_urls = 100;
$chatID = "";
$token = "";

function appendToFile($message, $filename = "errores.txt") {
    $time = '['.date("F j, Y, g:i a e O").']';
    error_log($time.' '.$message."\n", 3, $filename);
}

// Taken from https://stackoverflow.com/a/32875341 and edited
function sendMessage($message, $message_id = 0)
{
  global $chatID;
  global $token;

  if ($chatID == "" || $token == "") {
    appendToFile($message, "indexadas.txt");
    return;
  }

  $url = "";

  if ($message_id == 0) {
    $url = "https://api.telegram.org/bot$token/sendMessage?chat_id=$chatID";
  } else {
    $url = "https://api.telegram.org/bot$token/editMessageText?chat_id=$chatID&message_id=$message_id";
  }

  $url = $url . "&text=" . urlencode($message);
  $ch = curl_init();
  $optArray = array(
    CURLOPT_URL => $url,
    CURLOPT_RETURNTRANSFER => true
  );
  curl_setopt_array($ch, $optArray);
  $result = curl_exec($ch);
  curl_close($ch);
  return $result;
}

function savenum($num)
{
  global $curr_directory;
  file_put_contents("$curr_directory/lasturlposition.txt", $num);
}

function readnum()
{
  global $curr_directory;
  $filename = "$curr_directory/lasturlposition.txt";
  if (!file_exists($filename)) {
    return 0;
  }

  return trim(file_get_contents($filename));
}

function index($url)
{
  global $curr_directory;
  $client = new Google_Client();

  // service_account_file.json is the private key that you created for your service account.
  $client->setAuthConfig("$curr_directory/token.json");
  $client->addScope('https://www.googleapis.com/auth/indexing');

  // Get a Guzzle HTTP Client
  $httpClient = $client->authorize();
  $endpoint = 'https://indexing.googleapis.com/v3/urlNotifications:publish';

  // Define contents here. The structure of the content is described in the next step.
  $content = "{
    \"url\": \"$url\",
    \"type\": \"URL_UPDATED\"
  }";

  $response = $httpClient->post($endpoint, ['body' => $content]);
  return $response->getStatusCode();
}

try {
  if (!isset($curr_directory)) {
    throw new Exception("'curr_directory' variable is not set.");
  }

  if (!file_exists("$curr_directory/urls.txt")) {
    throw new Exception("The urls.txt file doesn't exist at $curr_directory");
  }

  if (!file_exists("$curr_directory/token.json")) {
    throw new Exception("The token.json file doesn't exist at $curr_directory");
  }

  $numUrlsToPublish = random_int($min_urls, $max_urls);
  $position_number = readnum();
  $data = [];
  $fails = [];

  $i = 0;
  $handle = fopen("$curr_directory/urls.txt", "r");
  if ($handle) {
    while (($line = fgets($handle)) !== false) {
      if ($i >= $position_number) {
        if ($i >= $position_number + $numUrlsToPublish) {
          break;
        }
        array_push($data, $line);
      }
      $i++;
    }

    fclose($handle);
  } else {
    throw new Exception("Error opening $curr_directory/urls.txt");
  }

  if ($chatID == "" || $token == "") {
    $message_id = 0;
  } else {
    $return_msg = json_decode(sendMessage("$numUrlsToPublish urls will be indexed. Beginning from $position_number."));
    $message_id = $return_msg->result->message_id;      
  }

  if (count($data) == 0) {
    sendMessage("[ALL INDEXED] All urls has been sent in $curr_directory", $message_id);
    return;
  }

  foreach ($data as $i => $url) {
    sleep(random_int(40, 60));

    sendMessage("[From $position_number -> ($i/$numUrlsToPublish)]\nIndexed: $url", $message_id);
    $response_code = index($url);
    if ($response_code != 200) {
      $fails[] = [$response_code, $url];
    }

    savenum($position_number + $i + 1);
  }

  $fails_n = count($fails);
  sendMessage("$curr_directory has started at $position_number, has sent $numUrlsToPublish URLs. It has got $fails_n failures.", $message_id);
  if ($fails_n > 0) {
    $error_msg = "";
    foreach ($fails as $i => $fail) {
      $error_msg .= "Error $i: $fail[0] at url $fail[1]\n";
    }
    throw new Exception($error_msg);
  }
} catch (\Throwable $th) {
  if ($chatID == "" || $token == "") {
    appendToFile($th->getMessage());
    return;
  } else {
    sendMessage($th->getMessage());    
  }
}
